# -*- coding: utf-8 -*-
from django.contrib import admin
from django.core.urlresolvers import reverse

from .models import MailingList, SubscriptionSettings, MassEmail


class MailingListAdmin(admin.ModelAdmin):
    list_display = ('name', 'user_should_be_agree', 'all_users')
    readonly_fields = ('or_list',)
    search_fields = ('name',)


def status_str(obj):
    return u"sent: {sent}, failed: {failed}, queued: {queued}".format(**obj.status)
status_str.short_description = 'Email queue status'


def template_link(obj):
    if obj.template:
        return (
            u'<a href="%s" target="_blank">EmailTemplate admin</a>. '
            u'Note: Editing template will not change queued emails'
            % reverse('admin:post_office_emailtemplate_change', args=[obj.template.id])
        )
    else:
        return 'N/A'

template_link.allow_tags = True
template_link.short_description = 'View template'


class MassEmailAdmin(admin.ModelAdmin):
    list_display = ('mailing_list', 'template', 'scheduled_time')
    readonly_fields = (status_str, template_link)
    fields = (
        status_str, 'mailing_list', 'template', template_link, 'scheduled_time', 'priority'
    )

admin.site.register(MailingList, MailingListAdmin)
admin.site.register(SubscriptionSettings)
admin.site.register(MassEmail, MassEmailAdmin)



# -*- coding: utf-8 -*-
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


try:
    from django.contrib.auth import get_user_model
except ImportError: # django < 1.5
    from django.contrib.auth.models import User
else:
    User = get_user_model()

user_orm_label = '%s.%s' % (User._meta.app_label, User._meta.object_name)
user_model_label = '%s.%s' % (User._meta.app_label, User._meta.module_name)
user_ptr_name = '%s_ptr' % User._meta.object_name.lower()
user_id_name = '%s_id' % User._meta.object_name

class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding model 'MailingList'
        db.create_table(u'mass_post_office_mailinglist', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('name', self.gf('django.db.models.fields.CharField')(max_length=255)),
            ('user_should_be_agree', self.gf('django.db.models.fields.BooleanField')(default=True)),
            ('all_users', self.gf('django.db.models.fields.BooleanField')(default=False)),
            ('or_list', self.gf('django.db.models.fields.TextField')(default='')),
        ))
        db.send_create_signal(u'mass_post_office', ['MailingList'])

        # Adding M2M table for field additional_users on 'MailingList'
        db.create_table(u'mass_post_office_mailinglist_additional_users', (
            ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True)),
            ('mailinglist', models.ForeignKey(orm[u'mass_post_office.mailinglist'], null=False)),
            (User._meta.object_name, models.ForeignKey(orm[user_model_label], null=False))
        ))
        db.create_unique(u'mass_post_office_mailinglist_additional_users', ['mailinglist_id', user_id_name])


    def backwards(self, orm):
        # Deleting model 'MailingList'
        db.delete_table(u'mass_post_office_mailinglist')

        # Removing M2M table for field additional_users on 'MailingList'
        db.delete_table('mass_post_office_mailinglist_additional_users')


    models = {
        user_model_label: {
        },
        u'mass_post_office.mailinglist': {
            'Meta': {'object_name': 'MailingList'},
            'additional_users': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['%s']" % user_orm_label, 'null': 'True', 'symmetrical': 'False'}),
            'all_users': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'or_list': ('django.db.models.fields.TextField', [], {'default': "''"}),
            'user_should_be_agree': ('django.db.models.fields.BooleanField', [], {'default': 'True'})
        },
        u'mass_post_office.subscriptionsettings': {
            'Meta': {'object_name': 'SubscriptionSettings'},
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'subscribed': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            User._meta.object_name: ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['%s']" % user_orm_label})
        }
    }

    complete_apps = ['mass_post_office']


# -*- coding: utf-8 -*-
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


try:
    from django.contrib.auth import get_user_model
except ImportError: # django < 1.5
    from django.contrib.auth.models import User
else:
    User = get_user_model()

user_orm_label = '%s.%s' % (User._meta.app_label, User._meta.object_name)
user_model_label = '%s.%s' % (User._meta.app_label, User._meta.module_name)
user_ptr_name = '%s_ptr' % User._meta.object_name.lower()

class Migration(SchemaMigration):

    def forwards(self, orm):

        # Changing field 'SubscriptionSettings.user'
        db.alter_column(u'mass_post_office_subscriptionsettings', 'user_id', self.gf('django.db.models.fields.related.OneToOneField')(to=orm[user_orm_label], unique=True))
        # Adding unique constraint on 'SubscriptionSettings', fields ['user']
        db.create_unique(u'mass_post_office_subscriptionsettings', ['user_id'])


    def backwards(self, orm):
        # Removing unique constraint on 'SubscriptionSettings', fields ['user']
        db.delete_unique(u'mass_post_office_subscriptionsettings', ['user_id'])


        # Changing field 'SubscriptionSettings.user'
        db.alter_column(u'mass_post_office_subscriptionsettings', 'user_id', self.gf('django.db.models.fields.related.ForeignKey')(to=orm[user_orm_label]))

    models = {
        user_model_label: {
        },
        u'mass_post_office.mailinglist': {
            'Meta': {'object_name': 'MailingList'},
            'additional_users': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['%s']" % user_orm_label, 'null': 'True', 'symmetrical': 'False'}),
            'all_users': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'or_list': ('django.db.models.fields.TextField', [], {'default': "''"}),
            'user_should_be_agree': ('django.db.models.fields.BooleanField', [], {'default': 'True'})
        },
        u'mass_post_office.subscriptionsettings': {
            'Meta': {'object_name': 'SubscriptionSettings'},
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'subscribed': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'user': ('django.db.models.fields.related.OneToOneField', [], {'to': "orm['%s']" % user_orm_label, 'unique': 'True'})
        }
    }

    complete_apps = ['mass_post_office']


# coding: utf-8
import zlib
import pickle
import hashlib
import urllib
import base64

import settings


def encode_data(data):
    """Turn `data` into a hash and an encoded string, suitable for use with
    `decode_data`.

    """
    text = base64.b64encode(zlib.compress(
        pickle.dumps(data, 0)
    )).replace('+', '.').replace('/', '-')
    m = hashlib.md5(settings.SECRET_KEY + text).hexdigest()[:12]
    return m, text


def decode_data(hashed, enc):
    """The inverse of `encode_data`.

    """
    text = urllib.unquote(enc)
    m = hashlib.md5(settings.SECRET_KEY + text).hexdigest()[:12]
    if m != hashed:
        raise ValueError("Bad hash!")
    text = text.replace('-', '/').replace('.', '+')
    data = pickle.loads(zlib.decompress(text.decode('base64')))
    return data



from django.contrib.sites.models import Site
from django import template
from django.core.urlresolvers import reverse

from ..utils import encode_data

register = template.Library()


@register.tag
def unsubscribe_link(parser, token):
    try:
        tag_name, obj = token.split_contents()
    except ValueError:
        raise template.TemplateSyntaxError(
            "%r tag requires a single argument" % token.contents.split()[0])
    return UnsubscribeLink(obj)


class UnsubscribeLink(template.Node):

    def __init__(self, obj):
        self.obj = template.Variable(obj)

    def render(self, context):
        user = self.obj.resolve(context)
        data = (user.username, user.email)
        hashed, data = encode_data(data)
        current_site = Site.objects.get_current()
        try:
            url = reverse('mass_post_office:unsubscribe', 
                kwargs={'hashed':hashed, 'data':data})
        except Exception:
            return u'http://%s' % current_site.domain
        # TODO https:// prefix
        return ''.join(['http://', current_site.domain, url])



# -*- coding: utf-8 -*-
import datetime
from django.core.exceptions import ValidationError
from django.test import TestCase

from ..models import MailingList, SubscriptionSettings, MassEmail

from .factories import UserFactory

from post_office.models import EmailTemplate


class TestModels(TestCase):

    def test_mailing_list(self):
        ml = MailingList(
            name='Some name',
            user_should_be_agree=True,
            all_users=True,
            or_list='')
        ml.save()
        self.assertEqual(str(ml), ml.name)
        # all users withoul user_should_be_agree -> Failure
        ml = MailingList(
            name='Some name',
            user_should_be_agree=False,
            all_users=True,
            or_list='')
        ml.clean()
        ml.save()
        self.assertTrue(ml.user_should_be_agree)
        # all users and or_list
        ml = MailingList(
            name='Some name',
            user_should_be_agree=True,
            all_users=True,
            or_list='{"some list": "123"}')
        ml.clean()
        ml.save()
        self.assertFalse(ml.or_list)
        # all users and additional users
        user = UserFactory()
        ml = MailingList(
            name='Some name',
            user_should_be_agree=True,
            all_users=True,
            or_list='')
        ml.save()
        ml.additional_users.add(user)
        ml.clean()
        ml.save()
        self.assertEqual(ml.additional_users.count(), 0)
        # test not valid json
        ml = MailingList(
            name='Some name',
            user_should_be_agree=True,
            all_users=True,
            or_list='{bad json}')
        is_exception = False
        try:
            ml.clean()
        except ValidationError:
            is_exception = True
        self.assertTrue(is_exception)

    def test_get_users_queryset(self):
        user1 = UserFactory(is_active=True)
        user2 = UserFactory(is_active=True)
        user3 = UserFactory(is_active=True)
        without_email = UserFactory(is_active=True)
        without_email.email = ''
        without_email.save()
        inactive_user = UserFactory(is_active=False)
        # all
        ml = MailingList(
            name='Some name',
            user_should_be_agree=False,
            all_users=True)
        ml.save()
        self.assertEqual(ml.get_users_queryset().count(), 3)
        # or list
        ml = MailingList(
            name='Some name',
            user_should_be_agree=False,
            all_users=False,
            or_list='[{"id": %d}]' % user2.id)
        ml.save()
        self.assertEqual(ml.get_users_queryset().count(), 1)
        self.assertEqual(ml.get_users_queryset()[0].id, user2.id)
        # additional users
        ml.additional_users.add(user1)
        self.assertEqual(ml.get_users_queryset().count(), 2)

    def test_get_emails_generator(self):
        user1 = UserFactory(is_active=True)
        user2 = UserFactory(is_active=True)
        ml = MailingList(
            name='Some name',
            user_should_be_agree=True,
            all_users=True)
        ml.save()
        generator = ml.get_emails_generator()
        emails = [e for e in generator]
        self.assertEqual(len(emails), 0)

        SubscriptionSettings.objects.create(user=user1, subscribed=True)
        SubscriptionSettings.objects.create(user=user2, subscribed=True)

        generator = ml.get_emails_generator()
        emails = [e for e in generator]
        self.assertEqual(len(emails), 2)
        self.assertIn(user1.email, emails)

    def test_mass_emails(self):
        priority = 2
        user1 = UserFactory(is_active=True)
        self.assertTrue(len(user1.first_name))
        self.assertTrue(len(user1.last_name))
        ml = MailingList(
            name='Some name',
            user_should_be_agree=False,
            all_users=True)
        ml.save()
        email_template = EmailTemplate.objects.create(
            name='welcome',
            subject='Welcome!',
            content='Hi {{ user.first_name }} {{ user.last_name }}!')
        today = datetime.datetime.today()
        from django.db.models import Q
        mass_emails_qs = Q(
            scheduled_time=today, priority=priority,
            mailing_list=ml, template=email_template)
        mass_emails = MassEmail.objects.filter(mass_emails_qs)
        self.assertEqual(len(mass_emails), 0)
        me = MassEmail(
            scheduled_time=today,
            priority=priority,
            mailing_list=ml,
            template=email_template
            )
        me.save()
        mass_emails = MassEmail.objects.filter(mass_emails_qs)
        self.assertEqual(len(mass_emails), 1)
        emails = me.emails.all()
        self.assertEqual(len(emails), 1)
        email = emails[0]
        self.assertEqual(user1.email, email.to)
        self.assertIn(user1.first_name, email.message)
        self.assertIn(user1.last_name, email.message)
        self.assertEqual(today, email.scheduled_time)
        self.assertEqual(priority, email.priority)



# -*- coding: utf-8 -*-
import datetime
from south.db import db
from south.v2 import DataMigration
from django.db import models

try:
    from django.contrib.auth import get_user_model
except ImportError: # django < 1.5
    from django.contrib.auth.models import User
else:
    User = get_user_model()

user_orm_label = '%s.%s' % (User._meta.app_label, User._meta.object_name)
user_model_label = '%s.%s' % (User._meta.app_label, User._meta.module_name)
user_ptr_name = '%s_ptr' % User._meta.object_name.lower()

class Migration(DataMigration):

    def forwards(self, orm):
        pass

    def backwards(self, orm):
        pass

    models = {
        user_model_label: {
        },
        u'mass_post_office.subscriptionsettings': {
            'Meta': {'object_name': 'SubscriptionSettings'},
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'subscribed': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['%s']" % user_orm_label})
        },
    }

    complete_apps = ['externalsite', 'mass_post_office']
    symmetrical = True



# -*- coding: utf-8 -*-
from django.shortcuts import render, redirect
from django.http import Http404
from django.core.urlresolvers import reverse
from django.conf import settings

from post_office import mail
from post_office.models import EmailTemplate

from .utils import decode_data
from .models import SubscriptionSettings


def unsubscribe(request, hashed, data):
    try:
        username, email = decode_data(hashed, data)
    except Exception:
        raise Http404
    try:
        subscription = SubscriptionSettings.objects.get(
            user__username=username,
            user__email=email)
    except SubscriptionSettings.DoesNotExist as e:
        raise Http404
    if not request.POST:
        context = {
            'user': subscription.user,
            'cancel_url': '/'
        }
        return render(request, 'mass_post_office/unsubscribe.html', context)
    if not 'is_unsubscribed' in request.POST:
        return redirect('/')
    subscription.subscribed = False
    subscription.save()
    try:
        mail.from_template(
            settings.DEFAULT_FROM_EMAIL,
            subscription.user.email,
            template=u'post_office/canceled_subscription',
            context={
                'user': subscription.user
            })
    except EmailTemplate.DoesNotExist as e:
        # we should process undefined EmailTemplate here
        # try to check 'post_migrate'
        pass
    return redirect(reverse('mass_post_office:unsubscribed'))


def unsubscribed(request):
    context = {}
    return render(request, 'mass_post_office/complete.html', context)



# -*- coding: utf-8 -*-
try:
    from south.signals import post_migrate
    from django.template.loader import find_template
    from django.dispatch import receiver

    from post_office import models as post_office_models

    from ..settings import EMAIL_TEMPLATES_SHOULD_BE_OVERRIDED

    DEFAULT_TEMPLATE_NAMES = [
        'post_office/canceled_subscription',
        'post_office/unsubscribe_page',
    ]

    @receiver(post_migrate, dispatch_uid='mass_post_office_models_post_migrate')
    def create_default_templates(sender, app, **kwargs):
        if app != 'post_office':
            return
        # populate django.template.loader.template_source_loaders
        try:
            find_template('hello')
        except Exception:
            pass

        from django.template.loader import template_source_loaders

        for template_name in DEFAULT_TEMPLATE_NAMES:
            # Pass overriding in order to project app has already created
            # its own email template
            if template_name in EMAIL_TEMPLATES_SHOULD_BE_OVERRIDED:
                continue
            if post_office_models.EmailTemplate.objects.filter(
                    name=template_name).exists():
                continue
            data = {'html_content': '', 'content': '', 'subject': ''}
            for loader in template_source_loaders:
                for key in data:
                    try:
                        data[key] = loader.load_template_source(
                            '%s_%s_default.txt' % (template_name, key)
                        )[0]
                    except Exception:
                        pass
            for key in data:
                if not data[key] and key != 'html_content':  # allow html content to be blank
                    template_full_name = '%s_%s_default.txt' % (template_name, key)
                    raise ValueError("Cant load template %s" % (template_full_name))
            post_office_models.EmailTemplate.objects.create(name=template_name, **data)

except ImportError:
    pass



# -*- coding: utf-8 -*-
from model_utils import Choices

from django.conf import settings
from django.contrib.auth import get_user_model
from django.core.exceptions import ValidationError
from django.db import models
from django.db.models import Q
from django.utils import simplejson
from django.utils.translation import ugettext_lazy as _

from post_office.models import EmailTemplate, Email, STATUS
from post_office.mail import from_template

USER_MODEL = getattr(settings, 'AUTH_USER_MODEL', 'auth.User')


class SubscriptionSettings(models.Model):
    user = models.OneToOneField(USER_MODEL)
    subscribed = models.BooleanField(default=False)

    def __unicode__(self):
        return '%s - %s' % (self.user.username, str(self.subscribed))

    class Meta:
        verbose_name_plural = "Subscription Settings"


class MailingList(models.Model):
    name = models.CharField(verbose_name='List name', max_length=255)
    user_should_be_agree = models.BooleanField(
        verbose_name=_('User should be agreed to receive mails'), default=True)
    all_users = models.BooleanField(
        verbose_name=_('All users'), default=False)
    or_list = models.TextField(verbose_name='json OR-list', default='')
    additional_users = models.ManyToManyField(
        USER_MODEL, verbose_name=_('Additional users'), null=True)

    def __unicode__(self):
        return self.name

    def clean(self):
        if self.or_list:
            try:
                simplejson.loads(self.or_list)
            except ValueError:
                raise ValidationError('OR-list json is not valid.')
        if self.all_users and not self.user_should_be_agree:
            self.user_should_be_agree = True
        if self.all_users and (self.or_list or (self.id and self.additional_users.count())):
            self.or_list = ''
            if self.id:
                for au in self.additional_users.all():
                    self.additional_users.remove(au)

    def get_users_queryset(self):
        if self.all_users:
            user_qs = get_user_model().objects.all()
        else:
            q = Q()
            if self.additional_users.exists():
                q |= Q(id__in=self.additional_users.values_list('id', flat=True))
            if self.or_list:
                for i in simplejson.loads(self.or_list):
                    q |= Q(**i)
            user_qs = get_user_model().objects.filter(q)
        if self.user_should_be_agree:
            user_qs = user_qs.filter(subscriptionsettings__subscribed=True)
        return (
            user_qs
            .filter(is_active=True, email__isnull=False)
            .exclude(email='').distinct()
        )

    def get_emails_generator(self):
        for user in self.get_users_queryset():
            yield user.email


class MassEmail(models.Model):
    PRIORITY_CHOICES = Choices(
        (0, 'low', _('low')),
        (1, 'medium', _('medium')),
        (2, 'high', _('high')),
        (3, 'now', _('now'))
        )

    mailing_list = models.ForeignKey(MailingList, verbose_name='Mailing List')
    template = models.ForeignKey(
        EmailTemplate,
        help_text='Template with access to `{{ user }}` variable',
        verbose_name='Template')
    emails = models.ManyToManyField(
        Email, verbose_name='Emails',
        null=True, blank=True)
    scheduled_time = models.DateTimeField(blank=True, null=True, db_index=True)
    priority = models.PositiveSmallIntegerField(
        choices=PRIORITY_CHOICES,
        blank=True, null=True, db_index=True)

    def __unicode__(self):
        return u'Mails for {self.mailing_list} with template {self.template}'.format(self=self)

    def save(self, *args, **kwargs):
        super(MassEmail, self).save(*args, **kwargs)
        for user in self.mailing_list.get_users_queryset():
            email = from_template(
                settings.DEFAULT_FROM_EMAIL,
                recipient=user.email,
                template=self.template,
                context={'user': user},
                scheduled_time=self.scheduled_time,
                priority=self.priority)
            email.save()
            self.emails.add(email)

    @property
    def status(self):
        result = {}
        for status in STATUS._fields:
            result[status] = self.emails.filter(status=getattr(STATUS, status)).count()

        return result


# signal processing
from django.db.models import signals

# by default, auto-add SubscriptionSettings objects for new Users
if getattr(settings, 'MASS_POST_OFFICE_SUBSCRIPTION_AUTO_ADD', True):
    def auto_add(sender, **kwargs):
        if sender == get_user_model() and kwargs['created']:
            user = kwargs.get('instance')
            SubscriptionSettings.objects.create(user=user)

    # TODO: try to only connect this to the User model. We can't use
    #       get_user_model() here - results in import loop.

    signals.post_save.connect(auto_add)
